//
//  TEPlainTextDocument.swift
//  TextEditDoc
//
//  Created by narita on 2017/12/25.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

class TERTFDDocument: NSDocument
{

    // MARK: - class methods

    override class var autosavesInPlace: Bool
    {
        return true
    }

    // MARK: - init methods

    override init()
    {
        super.init()
        // Add your subclass-specific initialization here.
        
        self.textStorage =  NSTextStorage()
    }

    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods

    
    override func fileWrapper(ofType typeName: String) throws -> FileWrapper
    {
        // "rtfd" 以外はサポートしない
        if !(NSWorkspace.shared.type(kUTTypeRTFD as String, conformsToType: typeName))
        {
            NSException(
                name: NSExceptionName.invalidArgumentException,
                reason: "\(typeName) is not a recognized document type.").raise()
        }

        // self.textStorageがnilならばエラー
        if self.textStorage == nil
        {
            NSException(
                name: NSExceptionName.objectNotAvailableException,
                reason: "self.textStorage is nil.").raise()
        }
        
        // textStragから"RegularFile"なfileWrapperを作る
        let theRange: NSRange  = NSMakeRange(0, self.textStorage!.length)
        
        // documentAttributesの引数には保存するdocument typeの設定が必要
        let theDocType = [NSAttributedString.DocumentAttributeKey.documentType : NSAttributedString.DocumentType.rtfd]
        let theResult: FileWrapper = try self.textStorage!.fileWrapper(from: theRange,
                                                                       documentAttributes: theDocType)
        
        return theResult
    }
    
    override func read(from fileWrapper: FileWrapper, ofType typeName: String) throws
    {
        // "rtfd" 以外はサポートしない
        if !(NSWorkspace.shared.type(kUTTypeRTFD as String, conformsToType: typeName))
        {
            NSException(
                name: NSExceptionName.invalidArgumentException,
                reason: "\(typeName) is not a recognized document type.").raise()
        }

        // バンドル形式のみサポートする
        if !fileWrapper.isDirectory
        {
            NSException(
                name: NSExceptionName.objectNotAvailableException,
                reason: "fileWrapper is not directory.").raise()
        }
        
        // fileWrapperからダイレクト生成するメソッドがあった。
        let theStorage = NSTextStorage(rtfdFileWrapper: fileWrapper, documentAttributes: nil)
        
        if theStorage == nil
        {
            NSException(
                name: NSExceptionName.objectNotAvailableException,
                reason: "fileWrapper is not RTFD.").raise()
        }
        
        self.textStorage = theStorage
    }

    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc var textStorage: NSTextStorage?

    // MARK: - Utility methods

    override func makeWindowControllers()
    {
        // Returns the Storyboard that contains your Document window.
        let storyboard = NSStoryboard(name: NSStoryboard.Name("TEWindowController"), bundle: nil)
        
        let windowController = storyboard.instantiateInitialController() as! TEWindowController

        self.addWindowController(windowController)
    }

}

