//
//  TESingleDocument.swift
//  TextEditDoc
//
//  Created by narita on 2017/12/25.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

class TESingleDocument: NSDocument
{

    // MARK: - class methods

    override class var autosavesInPlace: Bool
    {
        return true
    }

    // MARK: - init methods

    override init()
    {
        super.init()
        // Add your subclass-specific initialization here.
        
        self.textStorage =  NSTextStorage()
    }

    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    override func fileWrapper(ofType typeName: String) throws -> FileWrapper
    {
        // self.textStorageがnilならばエラー
        if self.textStorage == nil
        {
            NSException(
                name: NSExceptionName.objectNotAvailableException,
                reason: "self.textStorage is nil.").raise()
        }

        
        let theWorkspace = NSWorkspace.shared
        let theRange: NSRange  = NSMakeRange(0, self.textStorage!.length)
        
        if theWorkspace.type(kUTTypePlainText as String, conformsToType: typeName)
        {
            self.isRichText = false
            
            return
                try self.textStorage!.fileWrapper(from: theRange,
                                                  documentAttributes:
                    [NSAttributedString.DocumentAttributeKey.documentType : NSAttributedString.DocumentType.plain])

        }
        else if theWorkspace.type(kUTTypeRTF as String, conformsToType: typeName)
        {
            self.isRichText = true

            return
                try self.textStorage!.fileWrapper(from: theRange,
                                                  documentAttributes:
                    [NSAttributedString.DocumentAttributeKey.documentType : NSAttributedString.DocumentType.rtf])
        }
        else if theWorkspace.type(kUTTypeRTFD as String, conformsToType: typeName)
        {
            self.isRichText = true

            return
                try self.textStorage!.fileWrapper(from: theRange,
                                                  documentAttributes:
                    [NSAttributedString.DocumentAttributeKey.documentType : NSAttributedString.DocumentType.rtfd])
        }
        else if theWorkspace.type(kUTTypeHTML as String, conformsToType: typeName)
        {
            self.isRichText = true
            
            // fileWrapper(from:, documentAttributes:)を使うとなぜがパッケージ形式になってしまう。
            // NSDataを作ってから、FileWrapperにする
            let theData = try self.textStorage!.data(from: theRange,
                                                     documentAttributes:
                [NSAttributedString.DocumentAttributeKey.documentType : NSAttributedString.DocumentType.html])

            return FileWrapper(regularFileWithContents: theData)
        }
        else if theWorkspace.type(kUTTypeWebArchive as String, conformsToType: typeName)
        {
            self.isRichText = true
            
            return
                try self.textStorage!.fileWrapper(from: theRange,
                                                  documentAttributes:
                    [NSAttributedString.DocumentAttributeKey.documentType : NSAttributedString.DocumentType.webArchive])
        }
        else
        {
            NSException(
                name: NSExceptionName.invalidArgumentException,
                reason: "\(typeName) is not a recognized document type.").raise()
        }
        
        return FileWrapper()
    }
    
    override func read(from fileWrapper: FileWrapper, ofType typeName: String) throws
    {
        let theWorkspace = NSWorkspace.shared
        
        if theWorkspace.type(kUTTypePlainText as String, conformsToType: typeName)
        {
            self.textStorage = try NSTextStorage(data: fileWrapper.regularFileContents!,
                                                 options: [:],
                                                 documentAttributes: nil)
            self.isRichText = false
        }
        else if theWorkspace.type(kUTTypeRTF as String, conformsToType: typeName)
        {
            self.textStorage = try NSTextStorage(data: fileWrapper.regularFileContents!,
                                                 options: [:],
                                                 documentAttributes: nil)
            self.isRichText = true
        }
        else if theWorkspace.type(kUTTypeRTFD as String, conformsToType: typeName)
        {
            // このファイル形式は他のとtextStorageの生成方式が違う
            self.textStorage = NSTextStorage(rtfdFileWrapper: fileWrapper,
                                             documentAttributes: nil)
            self.isRichText = true
        }
        else if theWorkspace.type(kUTTypeHTML as String, conformsToType: typeName)
        {
            self.textStorage = try NSTextStorage(data: fileWrapper.regularFileContents!,
                                                 options: [:],
                                                 documentAttributes: nil)
            self.isRichText = true
        }
        else if theWorkspace.type(kUTTypeWebArchive as String, conformsToType: typeName)
        {
            self.textStorage = try NSTextStorage(data: fileWrapper.regularFileContents!,
                                                 options: [:],
                                                 documentAttributes: nil)
            self.isRichText = true
        }
        else
        {
            NSException(
                name: NSExceptionName.invalidArgumentException,
                reason: "\(typeName) is not a recognized document type.").raise()
        }
    }

    // これは、TextEditの[Document writableTypesForSaveOperation]をswift風に書いただけ
    @objc override func writableTypes(for saveOperation: NSDocument.SaveOperationType) -> [String]
    {
        var theResult = TESingleDocument.writableTypes
                
        if saveOperation == .saveAsOperation
        {
            // リッチテキストの時は、PlaneTextでは保存できないようにする
            if self.isRichText
            {
                // テキストは削除する。
                theResult.remove(at: theResult.index(of: kUTTypePlainText as String)! )
            }
            
            // 画像などの添付があれば、rtfdかwebArchiveでしか保存できない
            if self.textStorage!.containsAttachments(in: NSMakeRange(0, self.textStorage!.length))
            {
                theResult = [kUTTypeRTFD as String, kUTTypeWebArchive as String]
            }
        }
        
        return theResult
    }
    
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc var textStorage: NSTextStorage?
    @objc var isRichText: Bool = false

    
    
    
    // MARK: - Utility methods

    override func makeWindowControllers()
    {
        // Returns the Storyboard that contains your Document window.
        let storyboard = NSStoryboard(name: NSStoryboard.Name("TEWindowController"), bundle: nil)
        
        let windowController = storyboard.instantiateInitialController() as! TEWindowController

        self.addWindowController(windowController)
    }

}

